@@ -1,10 +1,12 @@
 /*****************************************************************************
  * demux.c
  *****************************************************************************
- * Copyright (C) 2004, 2008-2009 the VideoLAN team
- * $Id: demux.c 3 2009-05-11 13:19:16Z massiot $
+ * Copyright (C) 2004, 2008-2009 VideoLAN
+ * $Id: demux.c 69 2009-11-16 10:20:15Z md $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
+ *          Andy Gatward <a.j.gatward@reading.ac.uk>
+ *          Marian Ďurkovič <md@bts.sk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,12 +36,19 @@
 #include <dvbpsi/demux.h>
 #include <dvbpsi/pat.h>
 #include <dvbpsi/eit.h>
+#include <dvbpsi/sdt.h>
 #include <dvbpsi/dr.h>
 #include <dvbpsi/psi.h>
 
 /*****************************************************************************
  * Local declarations
  *****************************************************************************/
+
+#define PAT_PID        0x00
+#define SDT_PID        0x11
+#define EIT_PID        0x12
+#define TDT_PID        0x14
+
 typedef struct ts_pid_t
 {
     int i_refcount;
@@ -63,7 +72,10 @@ static sid_t **pp_sids = NULL;
 static int i_nb_sids = 0;
 
 static dvbpsi_handle p_pat_dvbpsi_handle;
+static dvbpsi_handle p_sdt_dvbpsi_handle;
+static dvbpsi_handle p_eit_dvbpsi_handle;
 static dvbpsi_pat_t *p_current_pat = NULL;
+static dvbpsi_sdt_t *p_current_sdt = NULL;
 static int i_demux_fd;
 
 /*****************************************************************************
@@ -85,11 +97,16 @@ static int SIDIsSelected( uint16_t i_sid );
 int PIDWouldBeSelected( dvbpsi_pmt_es_t *p_es );
 static int PMTNeedsDescrambling( dvbpsi_pmt_t *p_pmt );
 static void SendPAT( void );
+static void SendSDT( void );
 static void SendPMT( sid_t *p_sid );
 static void NewPAT( output_t *p_output );
+static void NewSDT( output_t *p_output );
 static void NewPMT( output_t *p_output );
 static void PATCallback( void *_unused, dvbpsi_pat_t *p_pat );
+static void SDTCallback( void *_unused, dvbpsi_sdt_t *p_sdt );
 static void PMTCallback( void *_unused, dvbpsi_pmt_t *p_pmt );
+static void PSITableCallback( void *_unused, dvbpsi_handle h_dvbpsi,
+                              uint8_t i_table_id, uint16_t i_extension );
 
 /*****************************************************************************
  * demux_Open
@@ -111,8 +128,19 @@ void demux_Open( void )
     if ( b_budget_mode )
         i_demux_fd = dvb_SetFilter(8192);
 
-    SetPID(0); /* PAT */
+    SetPID(PAT_PID); /* PAT */
     p_pat_dvbpsi_handle = dvbpsi_AttachPAT( PATCallback, NULL );
+
+    if( b_enable_epg )
+    {
+        SetPID(SDT_PID); /* SDT */
+        p_sdt_dvbpsi_handle = dvbpsi_AttachDemux( PSITableCallback, NULL );
+
+        SetPID(EIT_PID); /* EIT */
+        p_eit_dvbpsi_handle = dvbpsi_AttachDemux( PSITableCallback, NULL );
+
+        SetPID(TDT_PID); /* TDT */
+    }
 }
 
 /*****************************************************************************
@@ -159,12 +187,30 @@ static void demux_Handle( block_t *p_ts )
 
     if ( p_pids[i_pid].i_refcount )
     {
-        if ( i_pid == 0 )
+        if ( i_pid == PAT_PID )
         {
             dvbpsi_PushPacket( p_pat_dvbpsi_handle, p_ts->p_ts );
             if ( block_UnitStart( p_ts ) )
                 SendPAT();
         }
+        else if ( b_enable_epg && i_pid == EIT_PID )
+        {
+            dvbpsi_PushPacket( p_eit_dvbpsi_handle, p_ts->p_ts );
+        }
+        else if ( b_enable_epg && i_pid == SDT_PID )
+        {
+            dvbpsi_PushPacket( p_sdt_dvbpsi_handle, p_ts->p_ts );
+            if ( block_UnitStart( p_ts ) )
+                SendSDT();
+        }
+        else if ( b_enable_epg && i_pid == TDT_PID )
+        {
+            for ( i = 0; i < i_nb_outputs; i++ )
+            {
+                if ( pp_outputs[i]->i_maddr && pp_outputs[i]->p_sdt_section )
+                    output_Put( pp_outputs[i], p_ts );
+            }
+        }
         else
         {
             for ( i = 0; i < i_nb_sids; i++ )
@@ -197,7 +243,7 @@ static void demux_Handle( block_t *p_ts )
                     if ( pp_outputs[j]->i_sid == i_sid )
                     {
                         pp_outputs[j]->i_ref_timestamp = i_timestamp;
-                        pp_outputs[j]->i_ref_wallclock = i_wallclock;
+                        pp_outputs[j]->i_ref_wallclock = 0;
                     }
                 }
             }
@@ -271,7 +317,8 @@ void demux_Change( output_t *p_output, uint16_t i_sid,
     uint16_t *pi_wanted_pids, *pi_current_pids;
     int i_nb_wanted_pids, i_nb_current_pids;
     uint16_t i_old_sid = p_output->i_sid;
-    int b_change = 0;
+    int sid_change = ( i_sid != i_old_sid );
+    int pid_change = 0;
 
     if ( i_sid == p_output->i_sid && i_nb_pids == p_output->i_nb_pids &&
          (!i_nb_pids ||
@@ -282,57 +329,9 @@ void demux_Change( output_t *p_output, uint16_t i_sid,
     GetPIDS( &pi_current_pids, &i_nb_current_pids, p_output->i_sid,
              p_output->pi_pids, p_output->i_nb_pids );
 
-    for ( i = 0; i < i_nb_current_pids; i++ )
-    {
-        if ( pi_current_pids[i] != EMPTY_PID &&
-             !IsIn( pi_wanted_pids, i_nb_wanted_pids, pi_current_pids[i] ) )
-        {
-            StopPID( p_output, pi_current_pids[i] );
-            b_change = 1;
-        }
-    }
-
-    for ( i = 0; i < i_nb_wanted_pids; i++ )
-    {
-        if ( pi_wanted_pids[i] != EMPTY_PID &&
-             !IsIn( pi_current_pids, i_nb_current_pids, pi_wanted_pids[i] ) )
-        {
-            StartPID( p_output, pi_wanted_pids[i] );
-            b_change = 1;
-        }
-    }
-    free( pi_wanted_pids );
-    free( pi_current_pids );
-
-    if ( i_sid && p_output->i_sid != i_sid )
-    {
-        for ( i = 0; i < i_nb_sids; i++ )
-        {
-            if ( pp_sids[i]->i_sid == i_sid )
-            {
-                SetPID( pp_sids[i]->i_pmt_pid );
-
-                if ( pp_sids[i]->p_current_pmt != NULL )
-                {
-                    p_output->b_valid = 1;
-
-                    if ( i_ca_handle && !SIDIsSelected( i_sid )
-                          && PMTNeedsDescrambling( pp_sids[i]->p_current_pmt ) )
-                        en50221_AddPMT( pp_sids[i]->p_current_pmt );
-                    break;
-                }
-            }
-        }
-    }
-
-    p_output->i_sid = i_sid;
-    free( p_output->pi_pids );
-    p_output->pi_pids = malloc( sizeof(uint16_t) * i_nb_pids );
-    memcpy( p_output->pi_pids, pi_pids, sizeof(uint16_t) * i_nb_pids );
-    p_output->i_nb_pids = i_nb_pids;
-
-    if ( i_old_sid && i_old_sid != i_sid )
+    if ( sid_change && i_old_sid )
     {
+        p_output->i_sid = i_sid;
         for ( i = 0; i < i_nb_sids; i++ )
         {
             if ( pp_sids[i]->i_sid == i_old_sid )
@@ -348,14 +347,64 @@ void demux_Change( output_t *p_output, uint16_t i_sid,
         }
     }
 
-    if ( i_sid != i_old_sid )
+    for ( i = 0; i < i_nb_current_pids; i++ )
     {
-        NewPAT( p_output );
-        NewPMT( p_output );
+        if ( pi_current_pids[i] != EMPTY_PID &&
+             !IsIn( pi_wanted_pids, i_nb_wanted_pids, pi_current_pids[i] ) )
+        {
+            StopPID( p_output, pi_current_pids[i] );
+            pid_change = 1;
+        }
     }
-    else if ( b_change )
+
+    if ( sid_change &&
+         i_ca_handle && i_old_sid && SIDIsSelected( i_old_sid ) )
+    {
+        for ( i = 0; i < i_nb_sids; i++ )
+        {
+            if ( pp_sids[i]->i_sid == i_old_sid )
+            {
+                if ( pp_sids[i]->p_current_pmt != NULL 
+                      && PMTNeedsDescrambling( pp_sids[i]->p_current_pmt ) )
+                    en50221_UpdatePMT( pp_sids[i]->p_current_pmt );
+                break;
+            }
+        }
+    }
+
+    for ( i = 0; i < i_nb_wanted_pids; i++ )
+    {
+        if ( pi_wanted_pids[i] != EMPTY_PID &&
+             !IsIn( pi_current_pids, i_nb_current_pids, pi_wanted_pids[i] ) )
+        {
+            StartPID( p_output, pi_wanted_pids[i] );
+            pid_change = 1;
+        }
+    }
+
+    free( pi_wanted_pids );
+    free( pi_current_pids );
+
+    if ( sid_change && i_sid )
+    {
+        p_output->i_sid = i_old_sid;
+        for ( i = 0; i < i_nb_sids; i++ )
+        {
+            if ( pp_sids[i]->i_sid == i_sid )
+            {
+                SetPID( pp_sids[i]->i_pmt_pid );
+
+                if ( i_ca_handle && !SIDIsSelected( i_sid )
+                      && pp_sids[i]->p_current_pmt != NULL
+                      && PMTNeedsDescrambling( pp_sids[i]->p_current_pmt ) )
+                    en50221_AddPMT( pp_sids[i]->p_current_pmt );
+                break;
+            }
+        }
+    }
+
+    if ( i_ca_handle && i_sid && SIDIsSelected( i_sid ) )
     {
-        NewPMT( p_output );
         for ( i = 0; i < i_nb_sids; i++ )
         {
             if ( pp_sids[i]->i_sid == i_sid )
@@ -367,6 +416,21 @@ void demux_Change( output_t *p_output, uint16_t i_sid,
             }
         }
     }
+
+    p_output->i_sid = i_sid;
+    free( p_output->pi_pids );
+    p_output->pi_pids = malloc( sizeof(uint16_t) * i_nb_pids );
+    memcpy( p_output->pi_pids, pi_pids, sizeof(uint16_t) * i_nb_pids );
+    p_output->i_nb_pids = i_nb_pids;
+
+    if ( sid_change )
+    {
+        NewSDT( p_output );
+        NewPAT( p_output );
+        NewPMT( p_output );
+    }
+    else if ( pid_change )
+        NewPMT( p_output );
 }
 
 /*****************************************************************************
@@ -419,14 +483,12 @@ static void StartPID( output_t *p_output, uint16_t i_pid )
         }
 
         p_pids[i_pid].pp_outputs[j] = p_output;
+        SetPID( i_pid );
     }
-
-    SetPID( i_pid );
 }
 
 static void StopPID( output_t *p_output, uint16_t i_pid )
 {
-    int b_wanted = 0;
     int j;
 
     for ( j = 0; j < p_pids[i_pid].i_nb_outputs; j++ )
@@ -435,20 +497,14 @@ static void StopPID( output_t *p_output, uint16_t i_pid )
         {
             if ( p_pids[i_pid].pp_outputs[j] == p_output )
                 break;
-            b_wanted = 1;
         }
     }
 
-    if ( j == p_pids[i_pid].i_nb_outputs )
-        msg_Warn( NULL, "unselecting an unselected PID %d", i_pid );
-    else
+    if ( j != p_pids[i_pid].i_nb_outputs )
+    {
         p_pids[i_pid].pp_outputs[j] = NULL;
-
-    for ( j++; !b_wanted && j < p_pids[i_pid].i_nb_outputs; j++ )
-        if ( p_pids[i_pid].pp_outputs[j] != NULL )
-            b_wanted = 1;
-
-    UnsetPID( i_pid );
+        UnsetPID( i_pid );
+    }
 }
 
 /*****************************************************************************
@@ -567,7 +623,7 @@ static block_t *WritePSISection( dvbpsi_psi_section_t *p_section,
     {
         uint32_t i_copy = i_length > (184 - b_first) ? (184 - b_first) :
                           i_length;
-        int b_adaptation_field = (i_copy < (184 - b_first));
+        int i;
         block_t *p_ts;
 
         p_ts = *pp_last = block_New();
@@ -587,29 +643,19 @@ static block_t *WritePSISection( dvbpsi_psi_section_t *p_section,
         p_ts->p_ts[0] = 0x47;
         p_ts->p_ts[1] = ( b_first ? 0x40 : 0x00 ) | ( ( i_pid >> 8 ) & 0x1f );
         p_ts->p_ts[2] = i_pid & 0xff;
-        p_ts->p_ts[3] = ( b_adaptation_field ? 0x30 : 0x10 ) | *pi_cc;
+        p_ts->p_ts[3] = 0x10 | *pi_cc;
         (*pi_cc)++;
         *pi_cc &= 0xf;
 
-        if( b_adaptation_field )
-        {
-            int i_stuffing = 184 - i_copy - b_first;
-            int i;
-
-            p_ts->p_ts[4] = i_stuffing - 1;
-            if( i_stuffing > 1 )
-            {
-                p_ts->p_ts[5] = 0x00;
-                for( i = 6; i < 6 + i_stuffing - 2; i++ )
-                    p_ts->p_ts[i] = 0xff;
-            }
-        }
-
         if ( b_first )
-            p_ts->p_ts[188 - i_copy - 1] = 0; /* pointer */
+            p_ts->p_ts[4] = 0; /* pointer */
 
         /* copy payload */
-        memcpy( &p_ts->p_ts[188 - i_copy], p_data, i_copy );
+        memcpy( &p_ts->p_ts[4 + b_first], p_data, i_copy );
+
+        /* stuffing */
+        for( i = 4 + b_first + i_copy; i < 188; i++ )
+            p_ts->p_ts[i] = 0xff;
 
         b_first = 0;
         i_length -= i_copy;
@@ -629,11 +675,30 @@ static void SendPAT( void )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->p_pat_section != NULL )
+        if ( !pp_outputs[i]->i_maddr )
+            continue;
+
+        if ( pp_outputs[i]->p_pat_section == NULL &&
+             pp_outputs[i]->p_sdt_section != NULL && p_current_pat != NULL )
+        {
+            dvbpsi_pat_t pat;
+
+            if ( b_unique_tsid )
+                dvbpsi_InitPAT( &pat, pp_outputs[i]->i_ts_id,
+                                pp_outputs[i]->i_pat_version, 1 );
+            else
+                dvbpsi_InitPAT( &pat, p_current_pat->i_ts_id,
+                                pp_outputs[i]->i_pat_version, 1 );
+
+            pp_outputs[i]->p_pat_section = dvbpsi_GenPATSections( &pat, 0 );
+        }
+
+
+        if ( pp_outputs[i]->p_pat_section != NULL )
         {
             block_t *p_block;
 
-            p_block = WritePSISection( pp_outputs[i]->p_pat_section, 0,
+            p_block = WritePSISection( pp_outputs[i]->p_pat_section, PAT_PID,
                                        &pp_outputs[i]->i_pat_cc );
             while ( p_block != NULL )
             {
@@ -646,6 +711,32 @@ static void SendPAT( void )
     }
 }
 
+/*****************************************************************************
+ * SendSDT
+ *****************************************************************************/
+static void SendSDT( void )
+{
+    int i;
+
+    for ( i = 0; i < i_nb_outputs; i++ )
+    {
+        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->p_sdt_section != NULL )
+        {
+            block_t *p_block;
+
+            p_block = WritePSISection( pp_outputs[i]->p_sdt_section, SDT_PID,
+                                       &pp_outputs[i]->i_sdt_cc );
+            while ( p_block != NULL )
+            {
+                block_t *p_next = p_block->p_next;
+                p_block->i_refcount--;
+                output_Put( pp_outputs[i], p_block );
+                p_block = p_next;
+            }
+        }
+    }
+}
+
 /*****************************************************************************
  * SendPMT
  *****************************************************************************/
@@ -655,14 +746,52 @@ static void SendPMT( sid_t *p_sid )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == p_sid->i_sid
-              && pp_outputs[i]->p_pmt_section != NULL )
+        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == p_sid->i_sid )
+        {
+            if ( pp_outputs[i]->p_pmt_section != NULL )
+            {
+                block_t *p_block;
+
+                p_block = WritePSISection( pp_outputs[i]->p_pmt_section,
+                                           p_sid->i_pmt_pid,
+                                           &pp_outputs[i]->i_pmt_cc );
+                while ( p_block != NULL )
+                {
+                    block_t *p_next = p_block->p_next;
+                    p_block->i_refcount--;
+                    output_Put( pp_outputs[i], p_block );
+                    p_block = p_next;
+                }
+            }
+        }
+    }
+}
+
+/*****************************************************************************
+ * SendEIT
+ *****************************************************************************/
+static void SendEIT( dvbpsi_psi_section_t *p_section, uint16_t i_sid,
+                     uint8_t i_table_id )
+{
+    int i;
+
+
+    for( i = 0; i < i_nb_outputs; i++ )
+    {
+        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
         {
             block_t *p_block;
 
-            p_block = WritePSISection( pp_outputs[i]->p_pmt_section,
-                                       p_sid->i_pmt_pid,
-                                       &pp_outputs[i]->i_pmt_cc );
+            if( b_unique_tsid )
+            {
+                p_section->p_data[8]  = (pp_outputs[i]->i_ts_id >> 8) & 0xff;
+                p_section->p_data[9]  = pp_outputs[i]->i_ts_id & 0xff;
+                dvbpsi_BuildPSISection( p_section );
+            }
+
+            p_block = WritePSISection( p_section,
+                                       EIT_PID,
+                                       &pp_outputs[i]->i_eit_cc );
             while ( p_block != NULL )
             {
                 block_t *p_next = p_block->p_next;
@@ -695,23 +824,77 @@ static void NewPAT( output_t *p_output )
         if ( p_program->i_number == p_output->i_sid )
             break;
 
-    if ( p_program == NULL )
-    {
-        if ( p_output->b_valid )
-        {
-            msg_Warn( NULL, "can't find program %d !", p_output->i_sid );
-            p_output->b_valid = 0;
-        }
-        return;
-    }
+    if ( p_program == NULL ) return;
+
+    if ( b_unique_tsid )
+        dvbpsi_InitPAT( &pat, p_output->i_ts_id, p_output->i_pat_version, 1 );
+    else
+        dvbpsi_InitPAT( &pat, p_current_pat->i_ts_id, p_output->i_pat_version, 1 );
 
-    dvbpsi_InitPAT( &pat, p_output->i_sid, p_output->i_pat_version, 1 );
     dvbpsi_PATAddProgram( &pat, p_output->i_sid, p_program->i_pid );
 
     p_output->p_pat_section = dvbpsi_GenPATSections( &pat, 0 );
     dvbpsi_EmptyPAT( &pat );
 }
 
+/*****************************************************************************
+ * NewSDT
+ *****************************************************************************/
+static void NewSDT( output_t *p_output )
+{
+    dvbpsi_sdt_t sdt;
+    dvbpsi_sdt_service_t *p_service, *p_new_service ;
+    dvbpsi_descriptor_t *p_descriptor;
+
+    if ( p_output->p_sdt_section != NULL )
+        dvbpsi_DeletePSISections( p_output->p_sdt_section );
+    p_output->p_sdt_section = NULL;
+    p_output->i_sdt_version++;
+
+    if ( !p_output->i_sid ) return;
+    if ( p_current_sdt == NULL ) return;
+
+    for( p_service = p_current_sdt->p_first_service; p_service != NULL;
+         p_service = p_service->p_next )
+        if ( p_service->i_service_id == p_output->i_sid )
+            break;
+
+    if ( p_service == NULL )
+    {
+        if ( p_output->p_pat_section != NULL &&
+             p_output->p_pat_section->i_length == 9 )
+        {
+            dvbpsi_DeletePSISections( p_output->p_pat_section );
+            p_output->p_pat_section = NULL;
+            p_output->i_pat_version++;
+        }
+        return;
+    }
+
+    if ( b_unique_tsid )
+        dvbpsi_InitSDT( &sdt, p_output->i_ts_id,
+                        p_output->i_sdt_version, 1,
+                        p_current_sdt->i_network_id );
+    else
+        dvbpsi_InitSDT( &sdt, p_current_sdt->i_ts_id,
+                        p_output->i_sdt_version, 1,
+                        p_current_sdt->i_network_id );
+
+    p_new_service = dvbpsi_SDTAddService( &sdt,
+        p_service->i_service_id, p_service->b_eit_schedule,
+        p_service->b_eit_present, p_service->i_running_status, 0 );
+
+    for( p_descriptor = p_service->p_first_descriptor; p_descriptor != NULL;
+         p_descriptor = p_descriptor->p_next )
+        dvbpsi_SDTServiceAddDescriptor( p_new_service,
+                  p_descriptor->i_tag, p_descriptor->i_length,
+                  p_descriptor->p_data );
+
+    p_output->p_sdt_section = dvbpsi_GenSDTSections( &sdt );
+
+    dvbpsi_EmptySDT( &sdt );
+}
+
 /*****************************************************************************
  * NewPMT
  *****************************************************************************/
@@ -891,7 +1074,7 @@ static int PMTNeedsDescrambling( dvbpsi_pmt_t *p_pmt )
     for( p_dr = p_pmt->p_first_descriptor; p_dr != NULL; p_dr = p_dr->p_next )
         if( p_dr->i_tag == 0x9 )
             return 1;
- 
+
     for( p_es = p_pmt->p_first_es; p_es != NULL; p_es = p_es->p_next )
         for( p_dr = p_es->p_first_descriptor; p_dr != NULL;
              p_dr = p_dr->p_next )
@@ -914,6 +1097,52 @@ void demux_ResendCAPMTs( void )
             en50221_AddPMT( pp_sids[i]->p_current_pmt );
 }
 
+/*****************************************************************************
+ * DeleteProgram
+ *****************************************************************************/
+static void DeleteProgram( dvbpsi_pat_program_t *p_program )
+{
+    int i_pmt;
+
+    UnselectPSI( p_program->i_number, p_program->i_pid );
+
+    for ( i_pmt = 0; i_pmt < i_nb_sids; i_pmt++ )
+    {
+        if ( pp_sids[i_pmt]->i_sid == p_program->i_number )
+        {
+            dvbpsi_pmt_t *p_pmt = pp_sids[i_pmt]->p_current_pmt;
+
+            if ( p_pmt != NULL )
+            {
+                dvbpsi_pmt_es_t *p_es;
+
+                if ( i_ca_handle
+                     && SIDIsSelected( p_program->i_number )
+                     && PMTNeedsDescrambling( p_pmt ) )
+                    en50221_DeletePMT( p_pmt );
+
+                if ( p_pmt->i_pcr_pid != PADDING_PID
+                     && p_pmt->i_pcr_pid != pp_sids[i_pmt]->i_pmt_pid )
+                    UnselectPID( p_program->i_number, p_pmt->i_pcr_pid );
+
+                for( p_es = p_pmt->p_first_es; p_es != NULL;
+                     p_es = p_es->p_next )
+                {
+                    if ( PIDWouldBeSelected( p_es ) )
+                        UnselectPID( p_program->i_number, p_es->i_pid );
+                }
+
+                dvbpsi_DeletePMT( p_pmt );
+            }
+            pp_sids[i_pmt]->p_current_pmt = NULL;
+            pp_sids[i_pmt]->i_sid = 0;
+            pp_sids[i_pmt]->i_pmt_pid = 0;
+            dvbpsi_DetachPMT( pp_sids[i_pmt]->p_dvbpsi_handle );
+            break;
+        }
+    }
+}
+
 /*****************************************************************************
  * dvbpsi callbacks
  *****************************************************************************/
@@ -959,33 +1188,7 @@ static void PATCallback( void *_unused, dvbpsi_pat_t *p_pat )
 
             if ( p_old_program != NULL &&
                  p_old_program->i_pid != p_program->i_pid )
-            {
-                /* Delete old PID */
-                UnselectPSI( p_old_program->i_number,
-                             p_old_program->i_pid );
-
-                for ( i_pmt = 0; i_pmt < i_nb_sids; i_pmt++ )
-                {
-                    if ( pp_sids[i_pmt]->i_sid == p_program->i_number )
-                    {
-                        dvbpsi_pmt_t *p_old_pmt = pp_sids[i_pmt]->p_current_pmt;
-
-                        if ( p_old_pmt != NULL )
-                        {
-                            if ( i_ca_handle
-                                 && SIDIsSelected( p_old_program->i_number )
-                                 && PMTNeedsDescrambling( p_old_pmt ) )
-                                en50221_DeletePMT( p_old_pmt );
-                            dvbpsi_DeletePMT( p_old_pmt );
-                        }
-                        pp_sids[i_pmt]->p_current_pmt = NULL;
-                        pp_sids[i_pmt]->i_sid = 0;
-                        pp_sids[i_pmt]->i_pmt_pid = 0;
-                        dvbpsi_DetachPMT( pp_sids[i_pmt]->p_dvbpsi_handle );
-                        break;
-                    }
-                }
-            }
+                DeleteProgram( p_old_program );
         }
 
         SelectPSI( p_program->i_number, p_program->i_pid );
@@ -1027,36 +1230,11 @@ static void PATCallback( void *_unused, dvbpsi_pat_t *p_pat )
 
             if ( p_program == NULL )
             {
-                int i_pmt;
                 msg_Dbg( NULL, "  * removed number=%d pid=%d",
                          p_old_program->i_number,
                          p_old_program->i_pid );
 
-                UnselectPSI( p_old_program->i_number,
-                             p_old_program->i_pid );
-
-                for ( i_pmt = 0; i_pmt < i_nb_sids; i_pmt++ )
-                {
-                    if ( pp_sids[i_pmt]->i_sid == p_old_program->i_number )
-                    {
-                        dvbpsi_pmt_t *p_old_pmt = pp_sids[i_pmt]->p_current_pmt;
-
-                        if ( p_old_pmt != NULL )
-                        {
-                            if ( i_ca_handle
-                                 && SIDIsSelected( p_old_program->i_number )
-                                 && PMTNeedsDescrambling( p_old_pmt ) )
-                                en50221_DeletePMT( p_old_pmt );
-                            dvbpsi_DeletePMT( p_old_pmt );
-                        }
-                        pp_sids[i_pmt]->p_current_pmt = NULL;
-                        pp_sids[i_pmt]->i_sid = 0;
-                        pp_sids[i_pmt]->i_pmt_pid = 0;
-                        dvbpsi_DetachPMT( pp_sids[i_pmt]->p_dvbpsi_handle );
-                        break;
-                    }
-                }
-
+                DeleteProgram( p_old_program );
                 UpdatePAT( p_old_program->i_number );
             }
         }
@@ -1100,6 +1278,10 @@ static void PMTCallback( void *_unused, dvbpsi_pmt_t *p_pmt )
         return;
     }
 
+    if ( i_ca_handle && b_is_selected &&
+         !b_needs_descrambling && b_needed_descrambling )
+        en50221_DeletePMT( p_current_pmt );
+
     msg_Dbg( NULL, "new PMT program number=%d version=%d pid_pcr=%d",
              p_pmt->i_program_number, p_pmt->i_version, p_pmt->i_pcr_pid );
 
@@ -1154,9 +1336,86 @@ static void PMTCallback( void *_unused, dvbpsi_pmt_t *p_pmt )
             en50221_AddPMT( p_pmt );
         else if ( b_needs_descrambling && b_needed_descrambling )
             en50221_UpdatePMT( p_pmt );
-        else if ( !b_needs_descrambling && b_needed_descrambling )
-            en50221_DeletePMT( p_pmt );
     }
 
     UpdatePMT( p_pmt->i_program_number );
 }
+
+static void PSITableCallback( void *_unused, dvbpsi_handle h_dvbpsi,
+                              uint8_t i_table_id, uint16_t i_extension )
+{
+    /* EIT tables */
+
+    if ( i_table_id == 0x4e || ( i_table_id >= 0x50 && i_table_id <= 0x5f ) )
+    {
+        SendEIT( h_dvbpsi->p_current_section, i_extension, i_table_id );
+    }
+
+    /* SDT tables */
+
+    if ( i_table_id == 0x42 )
+    {
+        dvbpsi_AttachSDT( h_dvbpsi, i_table_id, i_extension, SDTCallback,
+                          NULL );
+    }
+}
+
+static void SDTCallback( void *_unused, dvbpsi_sdt_t *p_sdt )
+{
+    dvbpsi_sdt_t *p_old_sdt = p_current_sdt;
+    dvbpsi_sdt_service_t *p_srv;
+    dvbpsi_descriptor_t *p_dr;
+    int i;
+
+    if( p_current_sdt != NULL &&
+        ( !p_sdt->b_current_next ||
+          p_sdt->i_version == p_current_sdt->i_version ) )
+    {
+        dvbpsi_DeleteSDT( p_sdt );
+        return;
+    }
+
+    msg_Dbg( NULL, "new SDT ts_id=%d version=%d current_next=%d "
+             "network_id=%d",
+             p_sdt->i_ts_id, p_sdt->i_version, p_sdt->b_current_next,
+             p_sdt->i_network_id );
+
+    for( p_srv = p_sdt->p_first_service; p_srv; p_srv = p_srv->p_next )
+    {
+        msg_Dbg( NULL, "  * service id=%d eit schedule=%d present=%d "
+                 "running=%d free_ca=%d",
+                 p_srv->i_service_id, p_srv->b_eit_schedule,
+                 p_srv->b_eit_present, p_srv->i_running_status,
+                 p_srv->b_free_ca );
+
+        for( p_dr = p_srv->p_first_descriptor; p_dr; p_dr = p_dr->p_next )
+        {
+            if( p_dr->i_tag == 0x48 )
+            {
+                 dvbpsi_service_dr_t *pD = dvbpsi_DecodeServiceDr( p_dr );
+                 char str1[256];
+                 char str2[256];
+
+                 memcpy( str1, pD->i_service_provider_name,
+                         pD->i_service_provider_name_length );
+                 str1[pD->i_service_provider_name_length] = '\0';
+                 memcpy( str2, pD->i_service_name, pD->i_service_name_length );
+                 str2[pD->i_service_name_length] = '\0';
+ 
+                 msg_Dbg( NULL, "    - type=%d provider=%s name=%s",
+                          pD->i_service_type, str1, str2 );
+            }
+        }
+    }
+
+    p_current_sdt = p_sdt;
+
+    for ( i = 0; i < i_nb_outputs; i++ )
+    {
+        if ( pp_outputs[i]->i_maddr ) 
+            NewSDT( pp_outputs[i] );
+    }
+
+    if ( p_old_sdt != NULL )
+        dvbpsi_DeleteSDT( p_old_sdt );
+}
