@@ -1,8 +1,8 @@
 /*****************************************************************************
  * dvb.c
  *****************************************************************************
- * Copyright (C) 2008-2009 the VideoLAN team
- * $Id: dvb.c 5 2009-05-11 14:54:00Z massiot $
+ * Copyright (C) 2008-2009 VideoLAN
+ * $Id: dvb.c 72 2009-11-23 21:52:43Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -50,9 +50,10 @@
 /*****************************************************************************
  * Local declarations
  *****************************************************************************/
-#define FRONTEND_LOCK_TIMEOUT 10000000 /* 10 s */
+#define FRONTEND_LOCK_TIMEOUT 30000000 /* 30 s */
 #define COUNTER_WRAP 200 /* we make 200 read calls per second */
 #define MAX_READ_ONCE 50
+#define DVR_BUFFER_SIZE 40*188*1024 /* bytes */
 
 static int i_frontend, i_dvr;
 static fe_status_t i_last_status;
@@ -69,12 +70,6 @@ mtime_t i_ca_timeout = 0;
 static block_t *DVRRead( void );
 static void FrontendPoll( void );
 static void FrontendSet( void );
-#if DVB_API_VERSION < 5
-static void FrontendSetQPSK( struct dvb_frontend_parameters *p_fep );
-static void FrontendSetQAM( struct dvb_frontend_parameters *p_fep );
-static void FrontendSetOFDM( struct dvb_frontend_parameters *p_fep );
-static void FrontendSetATSC( struct dvb_frontend_parameters *p_fep );
-#endif
 
 /*****************************************************************************
  * dvb_Open
@@ -83,7 +78,9 @@ void dvb_Open( void )
 {
     char psz_tmp[128];
 
-    sprintf( psz_tmp, "/dev/dvb/adapter%d/frontend0", i_adapter );
+    msg_Dbg( NULL, "using linux-dvb API version %d", DVB_API_VERSION );
+
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/frontend%d", i_adapter, i_fenum );
     if( (i_frontend = open(psz_tmp, O_RDWR | O_NONBLOCK)) < 0 )
     {
         msg_Err( NULL, "opening device %s failed (%s)", psz_tmp,
@@ -93,7 +90,7 @@ void dvb_Open( void )
 
     FrontendSet();
 
-    sprintf( psz_tmp, "/dev/dvb/adapter%d/dvr0", i_adapter );
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/dvr%d", i_adapter, i_fenum );
 
     if( (i_dvr = open(psz_tmp, O_RDONLY)) < 0 )
     {
@@ -108,6 +105,12 @@ void dvb_Open( void )
                   strerror(errno) );
     }
 
+    if ( ioctl( i_dvr, DMX_SET_BUFFER_SIZE, DVR_BUFFER_SIZE ) < 0 )
+    {
+        msg_Warn( NULL, "couldn't set %s buffer size (%s)", psz_tmp,
+                 strerror(errno) );
+    }
+
     en50221_Init();
     i_ca_next_event = mdate() + i_ca_timeout;
 }
@@ -119,7 +122,6 @@ block_t *dvb_Read( void )
 {
     struct pollfd ufds[3];
     int i_ret, i_nb_fd = 2;
-    mtime_t i_current_date;
 
     memset( ufds, 0, sizeof(ufds) );
     ufds[0].fd = i_dvr;
@@ -141,19 +143,18 @@ block_t *dvb_Read( void )
             msg_Err( NULL, "poll error: %s", strerror(errno) );
         return NULL;
     }
-    i_current_date = mdate();
 
     if ( i_ca_handle && i_ca_type == CA_CI_LINK
-          && i_current_date > i_ca_next_event )
+          && mdate() > i_ca_next_event )
     {
         en50221_Poll();
-        i_ca_next_event = i_current_date + i_ca_timeout;
+        i_ca_next_event = mdate() + i_ca_timeout;
     }
 
     if ( ufds[1].revents )
         FrontendPoll();
 
-    if ( i_frontend_timeout && i_current_date > i_frontend_timeout )
+    if ( i_frontend_timeout && mdate() > i_frontend_timeout )
     {
         msg_Warn( NULL, "no lock, tuning again" );
         FrontendSet();
@@ -243,7 +244,7 @@ int dvb_SetFilter( uint16_t i_pid )
     char psz_tmp[128];
     int i_fd;
 
-    sprintf( psz_tmp, "/dev/dvb/adapter%d/demux0", i_adapter );
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/demux%d", i_adapter, i_fenum );
     if( (i_fd = open(psz_tmp, O_RDWR)) < 0 )
     {
         msg_Err( NULL, "DMXSetFilter: opening device failed (%s)",
@@ -372,7 +373,140 @@ static void FrontendPoll( void )
     }
 }
 
+static int FrontendDoDiseqc(void)
+{
+    fe_sec_voltage_t fe_voltage;
+    fe_sec_tone_mode_t fe_tone;
+    int bis_frequency;
+
+    switch ( i_voltage )
+    {
+        case 0: fe_voltage = SEC_VOLTAGE_OFF; break;
+        default:
+        case 13: fe_voltage = SEC_VOLTAGE_13; break;
+        case 18: fe_voltage = SEC_VOLTAGE_18; break;
+    }
+
+    fe_tone = b_tone ? SEC_TONE_ON : SEC_TONE_OFF;
+
+    /* Automatic mode. */
+    if ( i_frequency >= 950000 && i_frequency <= 2150000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in IF-band", i_frequency );
+        bis_frequency = i_frequency;
+    }
+    else if ( i_frequency >= 2500000 && i_frequency <= 2700000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in S-band", i_frequency );
+        bis_frequency = 3650000 - i_frequency;
+    }
+    else if ( i_frequency >= 3400000 && i_frequency <= 4200000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in C-band (lower)", i_frequency );
+        bis_frequency = 5150000 - i_frequency;
+    }
+    else if ( i_frequency >= 4500000 && i_frequency <= 4800000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in C-band (higher)", i_frequency );
+        bis_frequency = 5950000 - i_frequency;
+    }
+    else if ( i_frequency >= 10700000 && i_frequency < 11700000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in Ku-band (lower)",
+                 i_frequency );
+        bis_frequency = i_frequency - 9750000;
+    }
+    else if ( i_frequency >= 11700000 && i_frequency <= 13250000 )
+    {
+        msg_Dbg( NULL, "frequency %d is in Ku-band (higher)",
+                 i_frequency );
+        bis_frequency = i_frequency - 10600000;
+        fe_tone = SEC_TONE_ON;
+    }
+    else
+    {
+        msg_Err( NULL, "frequency %d is out of any known band",
+                 i_frequency );
+        exit(1);
+    }
+
+    /* Switch off continuous tone. */
+    if ( ioctl( i_frontend, FE_SET_TONE, SEC_TONE_OFF ) < 0 )
+    {
+        msg_Err( NULL, "FE_SET_TONE failed (%s)", strerror(errno) );
+        exit(1);
+    }
+
+    /* Configure LNB voltage. */
+    if ( ioctl( i_frontend, FE_SET_VOLTAGE, fe_voltage ) < 0 )
+    {
+        msg_Err( NULL, "FE_SET_VOLTAGE failed (%s)", strerror(errno) );
+        exit(1);
+    }
+
+    /* Wait for at least 15 ms. Currently 100 ms because of broken drivers. */
+    msleep(100000);
+
+    /* Diseqc */
+    if ( i_satnum > 0 && i_satnum < 5 )
+    {
+        /* digital satellite equipment control,
+         * specification is available from http://www.eutelsat.com/
+         */
+
+        struct dvb_diseqc_master_cmd cmd =
+            { {0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00}, 4};
+        cmd.msg[3] = 0xf0 /* reset bits */
+                          | ((i_satnum - 1) << 2)
+                          | (fe_voltage == SEC_VOLTAGE_13 ? 0 : 2)
+                          | (fe_tone == SEC_TONE_ON ? 1 : 0);
+
+        if( ioctl( i_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd ) < 0 )
+        {
+            msg_Err( NULL, "ioctl FE_SEND_MASTER_CMD failed (%s)",
+                     strerror(errno) );
+            exit(1);
+        }
+        msleep(100000); /* Should be 15 ms. */
+
+        /* Do it again just to be sure. */
+        if( ioctl( i_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd ) < 0 )
+        {
+            msg_Err( NULL, "ioctl FE_SEND_MASTER_CMD failed (%s)",
+                     strerror(errno) );
+            exit(1);
+        }
+        msleep(100000); /* Again, should be 15 ms */
+    }
+    else if ( i_satnum == 0xA || i_satnum == 0xB )
+    {
+        /* A or B simple diseqc ("diseqc-compatible") */
+        if( ioctl( i_frontend, FE_DISEQC_SEND_BURST,
+                   i_satnum == 0xB ? SEC_MINI_B : SEC_MINI_A ) < 0 )
+        {
+            msg_Err( NULL, "ioctl FE_SEND_BURST failed (%m)", strerror(errno) );
+            exit(1);
+        }
+        msleep(100000); /* ... */
+    }
+
+    else if ( ioctl( i_frontend, FE_SET_TONE, fe_tone ) < 0 )
+    {
+        msg_Err( NULL, "FE_SET_TONE failed (%s)", strerror(errno) );
+        exit(1);
+    }
+
+    msleep(100000); /* ... */
+
+    msg_Dbg( NULL, "configuring LNB to v=%d p=%d satnum=%x",
+             i_voltage, b_tone, i_satnum );
+    return bis_frequency;
+}
+
+#define DVBAPI_VERSION ((DVB_API_VERSION)*100+(DVB_API_VERSION_MINOR))
+
 #if DVB_API_VERSION >= 5
+
 /*****************************************************************************
  * GetModulation : helper function for both APIs
  *****************************************************************************/
@@ -408,68 +542,64 @@ static fe_modulation_t GetModulation(void)
  *****************************************************************************/
 /* S2API */
 static struct dtv_property dvbs_cmdargs[] = {
-	{ .cmd = DTV_FREQUENCY,       .u.data = 0 },
-	{ .cmd = DTV_MODULATION,      .u.data = QPSK },
-	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
-	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
-	{ .cmd = DTV_VOLTAGE,         .u.data = SEC_VOLTAGE_OFF },
-	{ .cmd = DTV_TONE,            .u.data = SEC_TONE_OFF },
-	{ .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
-	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS },
-	{ .cmd = DTV_TUNE },
+    { .cmd = DTV_FREQUENCY,       .u.data = 0 },
+    { .cmd = DTV_MODULATION,      .u.data = QPSK },
+    { .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
+    { .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
+    { .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
+    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS },
+    { .cmd = DTV_TUNE },
 };
 static struct dtv_properties dvbs_cmdseq = {
-	.num = sizeof(dvbs_cmdargs)/sizeof(struct dtv_property),
-	.props = dvbs_cmdargs
+    .num = sizeof(dvbs_cmdargs)/sizeof(struct dtv_property),
+    .props = dvbs_cmdargs
 };
 
 static struct dtv_property dvbs2_cmdargs[] = {
-	{ .cmd = DTV_FREQUENCY,       .u.data = 0 },
-	{ .cmd = DTV_MODULATION,      .u.data = PSK_8 },
-	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
-	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
-	{ .cmd = DTV_VOLTAGE,         .u.data = SEC_VOLTAGE_OFF },
-	{ .cmd = DTV_TONE,            .u.data = SEC_TONE_OFF },
-	{ .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
-	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS2 },
-	{ .cmd = DTV_PILOT,           .u.data = PILOT_AUTO },
-	{ .cmd = DTV_ROLLOFF,         .u.data = ROLLOFF_AUTO },
-	{ .cmd = DTV_TUNE },
+    { .cmd = DTV_FREQUENCY,       .u.data = 0 },
+    { .cmd = DTV_MODULATION,      .u.data = PSK_8 },
+    { .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
+    { .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
+    { .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
+    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS2 },
+    { .cmd = DTV_PILOT,           .u.data = PILOT_AUTO },
+    { .cmd = DTV_ROLLOFF,         .u.data = ROLLOFF_AUTO },
+    { .cmd = DTV_TUNE },
 };
 static struct dtv_properties dvbs2_cmdseq = {
-	.num = sizeof(dvbs2_cmdargs)/sizeof(struct dtv_property),
-	.props = dvbs2_cmdargs
+    .num = sizeof(dvbs2_cmdargs)/sizeof(struct dtv_property),
+    .props = dvbs2_cmdargs
 };
 
 static struct dtv_property dvbc_cmdargs[] = {
-	{ .cmd = DTV_FREQUENCY,       .u.data = 0 },
-	{ .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
-	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
-	{ .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
-	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBC_ANNEX_AC },
-	{ .cmd = DTV_TUNE },
+    { .cmd = DTV_FREQUENCY,       .u.data = 0 },
+    { .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
+    { .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
+    { .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
+    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBC_ANNEX_AC },
+    { .cmd = DTV_TUNE },
 };
 static struct dtv_properties dvbc_cmdseq = {
-	.num = sizeof(dvbc_cmdargs)/sizeof(struct dtv_property),
-	.props = dvbc_cmdargs
+    .num = sizeof(dvbc_cmdargs)/sizeof(struct dtv_property),
+    .props = dvbc_cmdargs
 };
 
 static struct dtv_property dvbt_cmdargs[] = {
-	{ .cmd = DTV_FREQUENCY,       .u.data = 0 },
-	{ .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
-	{ .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
-	{ .cmd = DTV_BANDWIDTH_HZ,    .u.data = 8000000 },
-	{ .cmd = DTV_CODE_RATE_HP,    .u.data = FEC_AUTO },
-	{ .cmd = DTV_CODE_RATE_LP,    .u.data = FEC_AUTO },
-	{ .cmd = DTV_GUARD_INTERVAL,  .u.data = GUARD_INTERVAL_AUTO },
-	{ .cmd = DTV_TRANSMISSION_MODE,.u.data = TRANSMISSION_MODE_AUTO },
-	{ .cmd = DTV_HIERARCHY,       .u.data = HIERARCHY_AUTO },
-	{ .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBT },
-	{ .cmd = DTV_TUNE },
+    { .cmd = DTV_FREQUENCY,       .u.data = 0 },
+    { .cmd = DTV_MODULATION,      .u.data = QAM_AUTO },
+    { .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
+    { .cmd = DTV_BANDWIDTH_HZ,    .u.data = 8000000 },
+    { .cmd = DTV_CODE_RATE_HP,    .u.data = FEC_AUTO },
+    { .cmd = DTV_CODE_RATE_LP,    .u.data = FEC_AUTO },
+    { .cmd = DTV_GUARD_INTERVAL,  .u.data = GUARD_INTERVAL_AUTO },
+    { .cmd = DTV_TRANSMISSION_MODE,.u.data = TRANSMISSION_MODE_AUTO },
+    { .cmd = DTV_HIERARCHY,       .u.data = HIERARCHY_AUTO },
+    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBT },
+    { .cmd = DTV_TUNE },
 };
 static struct dtv_properties dvbt_cmdseq = {
-	.num = sizeof(dvbt_cmdargs)/sizeof(struct dtv_property),
-	.props = dvbt_cmdargs
+    .num = sizeof(dvbt_cmdargs)/sizeof(struct dtv_property),
+    .props = dvbt_cmdargs
 };
 
 #define FREQUENCY 0
@@ -477,9 +607,6 @@ static struct dtv_properties dvbt_cmdseq = {
 #define INVERSION 2
 #define SYMBOL_RATE 3
 #define BANDWIDTH 3
-#define VOLTAGE 4
-#define TONE 5
-#define INNER_FEC 6
 
 static void FrontendSet( void )
 {
@@ -526,63 +653,12 @@ static void FrontendSet( void )
         }
         else
             p = &dvbs_cmdseq;
+
         p->props[SYMBOL_RATE].u.data = i_srate;
+        p->props[FREQUENCY].u.data = FrontendDoDiseqc();
 
-        switch ( i_voltage )
-        {
-            case 0: p->props[VOLTAGE].u.data = SEC_VOLTAGE_OFF; break;
-            default:
-            case 13: p->props[VOLTAGE].u.data = SEC_VOLTAGE_13; break;
-            case 18: p->props[VOLTAGE].u.data = SEC_VOLTAGE_18; break;
-        }
-
-        p->props[TONE].u.data = b_tone ? SEC_TONE_ON : SEC_TONE_OFF;
-
-        /* Automatic mode. */
-        if ( i_frequency >= 950000 && i_frequency <= 2150000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in IF-band", i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency;
-        }
-        else if ( i_frequency >= 2500000 && i_frequency <= 2700000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in S-band", i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency - 3650000;
-        }
-        else if ( i_frequency >= 3400000 && i_frequency <= 4200000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in C-band (lower)",
-                     i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency - 5150000;
-        }
-        else if ( i_frequency >= 4500000 && i_frequency <= 4800000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in C-band (higher)",
-                     i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency - 5950000;
-        }
-        else if ( i_frequency >= 10700000 && i_frequency < 11700000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in Ku-band (lower)",
-                     i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency - 9750000;
-        }
-        else if ( i_frequency >= 11700000 && i_frequency <= 13250000 )
-        {
-            msg_Dbg( NULL, "frequency %d is in Ku-band (higher)",
-                     i_frequency );
-            p->props[FREQUENCY].u.data = i_frequency - 10600000;
-            p->props[TONE].u.data = SEC_TONE_ON;
-        }
-        else
-        {
-            msg_Err( NULL, "frequency %d is out of any known band",
-                     i_frequency );
-            exit(1);
-        }
-
-        msg_Dbg( NULL, "tuning QPSK frontend to f=%d srate=%d v=%d p=%d modulation=%s",
-                 i_frequency, i_srate, i_voltage, b_tone,
+        msg_Dbg( NULL, "tuning QPSK frontend to f=%d srate=%d modulation=%s",
+                 i_frequency, i_srate,
                  psz_modulation == NULL ? "legacy" : psz_modulation );
         break;
 
@@ -627,17 +703,59 @@ static void FrontendSet( void )
     switch ( info.type )
     {
     case FE_OFDM:
-        FrontendSetOFDM( &fep );
+        fep.frequency = i_frequency;
+        fep.inversion = INVERSION_AUTO;
+
+        switch ( i_bandwidth )
+        {
+            case 6: fep.u.ofdm.bandwidth = BANDWIDTH_6_MHZ; break;
+            case 7: fep.u.ofdm.bandwidth = BANDWIDTH_7_MHZ; break;
+            default:
+            case 8: fep.u.ofdm.bandwidth = BANDWIDTH_8_MHZ; break;
+        }
+
+        fep.u.ofdm.code_rate_HP = FEC_AUTO;
+        fep.u.ofdm.code_rate_LP = FEC_AUTO;
+        fep.u.ofdm.constellation = QAM_AUTO;
+        fep.u.ofdm.transmission_mode = TRANSMISSION_MODE_AUTO;
+        fep.u.ofdm.guard_interval = GUARD_INTERVAL_AUTO;
+        fep.u.ofdm.hierarchy_information = HIERARCHY_AUTO;
+
+        msg_Dbg( NULL, "tuning OFDM frontend to f=%d, bandwidth=%d",
+                 i_frequency, i_bandwidth );
         break;
+
     case FE_QAM:
-        FrontendSetQAM( &fep );
+        fep.frequency = i_frequency;
+        fep.inversion = INVERSION_AUTO;
+        fep.u.qam.symbol_rate = i_srate;
+        fep.u.qam.fec_inner = FEC_AUTO;
+        fep.u.qam.modulation = QAM_AUTO;
+
+        msg_Dbg( NULL, "tuning QAM frontend to f=%d, srate=%d", i_frequency,
+                 i_srate );
         break;
+
     case FE_QPSK:
-        FrontendSetQPSK( &fep );
+        fep.inversion = INVERSION_AUTO;
+        fep.u.qpsk.symbol_rate = i_srate;
+        fep.u.qpsk.fec_inner = FEC_AUTO;
+        fep.frequency = FrontendDoDiseqc();
+
+        msg_Dbg( NULL, "tuning QPSK frontend to f=%d, srate=%d",
+                 i_frequency, i_srate );
         break;
+
+#if DVBAPI_VERSION >= 301
     case FE_ATSC:
-        FrontendSetATSC( &fep );
+        fep.frequency = i_frequency;
+
+        fep.u.vsb.modulation = QAM_AUTO;
+
+        msg_Dbg( NULL, "tuning ATSC frontend to f=%d", i_frequency );
         break;
+#endif
+
     default:
         msg_Err( NULL, "unknown frontend type %d", info.type );
         exit(1);
@@ -663,156 +781,6 @@ static void FrontendSet( void )
     i_frontend_timeout = mdate() + FRONTEND_LOCK_TIMEOUT;
 }
 
-/*****************************************************************************
- * FrontendSetQPSK
- *****************************************************************************/
-static void FrontendSetQPSK( struct dvb_frontend_parameters *p_fep )
-{
-    fe_sec_voltage_t fe_voltage;
-    fe_sec_tone_mode_t fe_tone;
-
-    p_fep->inversion = INVERSION_AUTO;
-    p_fep->u.qpsk.symbol_rate = i_srate;
-    p_fep->u.qpsk.fec_inner = FEC_AUTO;
-
-    switch ( i_voltage )
-    {
-        case 0: fe_voltage = SEC_VOLTAGE_OFF; break;
-        default:
-        case 13: fe_voltage = SEC_VOLTAGE_13; break;
-        case 18: fe_voltage = SEC_VOLTAGE_18; break;
-    }
-
-    fe_tone = b_tone ? SEC_TONE_ON : SEC_TONE_OFF;
-
-    /* Automatic mode. */
-    if ( i_frequency >= 950000 && i_frequency <= 2150000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in IF-band", i_frequency );
-        p_fep->frequency = i_frequency;
-    }
-    else if ( i_frequency >= 2500000 && i_frequency <= 2700000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in S-band", i_frequency );
-        p_fep->frequency = i_frequency - 3650000;
-    }
-    else if ( i_frequency >= 3400000 && i_frequency <= 4200000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in C-band (lower)",
-                 i_frequency );
-        p_fep->frequency = i_frequency - 5150000;
-    }
-    else if ( i_frequency >= 4500000 && i_frequency <= 4800000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in C-band (higher)",
-                 i_frequency );
-        p_fep->frequency = i_frequency - 5950000;
-    }
-    else if ( i_frequency >= 10700000 && i_frequency < 11700000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in Ku-band (lower)",
-                 i_frequency );
-        p_fep->frequency = i_frequency - 9750000;
-    }
-    else if ( i_frequency >= 11700000 && i_frequency <= 13250000 )
-    {
-        msg_Dbg( NULL, "frequency %d is in Ku-band (higher)",
-                 i_frequency );
-        p_fep->frequency = i_frequency - 10600000;
-        fe_tone = SEC_TONE_ON;
-    }
-    else
-    {
-        msg_Err( NULL, "frequency %d is out of any known band",
-                 i_frequency );
-        exit(1);
-    }
-
-    /* Switch off continuous tone. */
-    if ( ioctl( i_frontend, FE_SET_TONE, SEC_TONE_OFF ) < 0 )
-    {
-        msg_Err( NULL, "FE_SET_TONE failed (%s)", strerror(errno) );
-        exit(1);
-    }
-
-    /* Configure LNB voltage. */
-    if ( ioctl( i_frontend, FE_SET_VOLTAGE, fe_voltage ) < 0 )
-    {
-        msg_Err( NULL, "FE_SET_VOLTAGE failed (%s)", strerror(errno) );
-        exit(1);
-    }
-
-    /* Wait for at least 15 ms. */
-    msleep(15000);
-
-    if ( ioctl( i_frontend, FE_SET_TONE, fe_tone ) < 0 )
-    {
-        msg_Err( NULL, "FE_SET_TONE failed (%s)", strerror(errno) );
-        exit(1);
-    }
-
-    /* TODO: Diseqc */
-
-    msleep(50000);
-
-    msg_Dbg( NULL, "tuning QPSK frontend to f=%d, srate=%d, v=%d, p=%d",
-             i_frequency, i_srate, i_voltage, b_tone );
-}
-
-/*****************************************************************************
- * FrontendSetQAM
- *****************************************************************************/
-static void FrontendSetQAM( struct dvb_frontend_parameters *p_fep )
-{
-    p_fep->frequency = i_frequency;
-    p_fep->inversion = INVERSION_AUTO;
-    p_fep->u.qam.symbol_rate = i_srate;
-    p_fep->u.qam.fec_inner = FEC_AUTO;
-    p_fep->u.qam.modulation = QAM_AUTO;
-
-    msg_Dbg( NULL, "tuning QAM frontend to f=%d, srate=%d", i_frequency,
-             i_srate );
-}
-
-/*****************************************************************************
- * FrontendSetOFDM
- *****************************************************************************/
-static void FrontendSetOFDM( struct dvb_frontend_parameters *p_fep )
-{
-    p_fep->frequency = i_frequency;
-    p_fep->inversion = INVERSION_AUTO;
-
-    switch ( i_bandwidth )
-    {
-        case 6: p_fep->u.ofdm.bandwidth = BANDWIDTH_6_MHZ; break;
-        case 7: p_fep->u.ofdm.bandwidth = BANDWIDTH_7_MHZ; break;
-        default:
-        case 8: p_fep->u.ofdm.bandwidth = BANDWIDTH_8_MHZ; break;
-    }
-
-    p_fep->u.ofdm.code_rate_HP = FEC_AUTO;
-    p_fep->u.ofdm.code_rate_LP = FEC_AUTO;
-    p_fep->u.ofdm.constellation = QAM_AUTO;
-    p_fep->u.ofdm.transmission_mode = TRANSMISSION_MODE_AUTO;
-    p_fep->u.ofdm.guard_interval = GUARD_INTERVAL_AUTO;
-    p_fep->u.ofdm.hierarchy_information = HIERARCHY_AUTO;
-
-    msg_Dbg( NULL, "tuning OFDM frontend to f=%d, bandwidth=%d", i_frequency,
-             i_bandwidth );
-}
-
-/*****************************************************************************
- * FrontendSetATSC
- *****************************************************************************/
-static void FrontendSetATSC( struct dvb_frontend_parameters *p_fep )
-{
-    p_fep->frequency = i_frequency;
-
-    p_fep->u.vsb.modulation = QAM_AUTO;
-
-    msg_Dbg( NULL, "tuning ATSC frontend to f=%d", i_frequency );
-}
-
 #endif /* S2API */
 
 /*****************************************************************************
