@@ -2,8 +2,8 @@
  * en50221.c : implementation of the transport, session and applications
  * layers of EN 50 221
  *****************************************************************************
- * Copyright (C) 2004-2005 the VideoLAN team
- * $Id: en50221.c 3 2009-05-11 13:19:16Z massiot $
+ * Copyright (C) 2004-2005 VideoLAN
+ * $Id: en50221.c 51 2009-11-03 22:23:57Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  * Based on code from libdvbci Copyright (C) 2000 Klaus Schmidinger
@@ -37,6 +37,7 @@
 #include <arpa/inet.h>
 #include <poll.h>
 #include <errno.h>
+#include <time.h>
 
 /* DVB Card Drivers */
 #include <linux/dvb/version.h>
@@ -66,6 +67,7 @@
 #undef DEBUG_TPDU
 #define HLCI_WAIT_CAM_READY 0
 #define CAM_PROG_MAX MAX_PROGRAMS
+#define CAPMT_WAIT 100 /* ms */
 
 typedef struct en50221_session_t
 {
@@ -160,7 +162,7 @@ static uint8_t *SetLength( uint8_t *p_data, int i_length )
  * Transport layer
  */
 
-#define MAX_TPDU_SIZE  2048
+#define MAX_TPDU_SIZE  4096
 #define MAX_TPDU_DATA  (MAX_TPDU_SIZE - 4)
 
 #define DATA_INDICATOR 0x80
@@ -812,10 +814,8 @@ static int APDUSend( access_t * p_access, int i_session_id, int i_tag,
         }
         else
         {
-            char *psz_hex;
             ca_msg.length = i_size + p - p_apdu;
             if ( i_size == 0 ) ca_msg.length=3;
-            psz_hex = (char*)malloc( ca_msg.length*3 + 1);
             memcpy( ca_msg.msg, p_apdu, i_size + p - p_apdu );
             i_ret = ioctl(i_ca_handle, CA_SEND_MSG, &ca_msg );
             if ( i_ret < 0 )
@@ -949,19 +949,21 @@ static void ApplicationInformationOpen( access_t * p_access, int i_session_id )
  * Conditional Access
  */
 
-#define MAX_CASYSTEM_IDS 16
+#define MAX_CASYSTEM_IDS 64
 
 typedef struct
 {
     uint16_t pi_system_ids[MAX_CASYSTEM_IDS + 1];
 
     int i_selected_programs;
+    int b_high_level;
 } system_ids_t;
 
 static int CheckSystemID( system_ids_t *p_ids, uint16_t i_id )
 {
     int i = 0;
-    if( !p_ids ) return 1;
+    if( !p_ids ) return 0;
+    if( p_ids->b_high_level ) return 1;
 
     while ( p_ids->pi_system_ids[i] )
     {
@@ -1154,6 +1156,14 @@ static uint8_t *CAPMTBuild( access_t * p_access, int i_session_id,
         }
     }
 
+    if ( *pi_capmt_size <= 7 + i_cad_program_size )
+    {
+        msg_Dbg( p_access, "CAPMT not needed, no ES selected" );
+        free( p_capmt );
+        *pi_capmt_size = 0;
+        return NULL;
+    }
+
     return p_capmt;
 }
 
@@ -1174,7 +1184,10 @@ static void CAPMTFirst( access_t * p_access, int i_session_id,
                           &i_capmt_size );
 
     if ( i_capmt_size )
+    {
         APDUSend( p_access, i_session_id, AOT_CA_PMT, p_capmt, i_capmt_size );
+        free( p_capmt );
+    }
 }
 
 /*****************************************************************************
@@ -1201,6 +1214,8 @@ static void CAPMTAdd( access_t * p_access, int i_session_id,
         return;
     }
 
+    if( b_slow_cam )
+        msleep( CAPMT_WAIT * 1000 );
 
     msg_Dbg( p_access, "adding CAPMT for SID %d on session %d",
              p_pmt->i_program_number, i_session_id );
@@ -1210,7 +1225,10 @@ static void CAPMTAdd( access_t * p_access, int i_session_id,
                           &i_capmt_size );
 
     if ( i_capmt_size )
+    {
         APDUSend( p_access, i_session_id, AOT_CA_PMT, p_capmt, i_capmt_size );
+        free( p_capmt );
+    }
 }
 
 /*****************************************************************************
@@ -1230,7 +1248,10 @@ static void CAPMTUpdate( access_t * p_access, int i_session_id,
                           &i_capmt_size );
 
     if ( i_capmt_size )
+    {
         APDUSend( p_access, i_session_id, AOT_CA_PMT, p_capmt, i_capmt_size );
+        free( p_capmt );
+    }
 }
 
 /*****************************************************************************
@@ -1253,7 +1274,10 @@ static void CAPMTDelete( access_t * p_access, int i_session_id,
                           &i_capmt_size );
 
     if ( i_capmt_size )
+    {
         APDUSend( p_access, i_session_id, AOT_CA_PMT, p_capmt, i_capmt_size );
+        free( p_capmt );
+    }
 }
 
 /*****************************************************************************
@@ -1957,6 +1981,8 @@ void en50221_Reset( void )
     else
     {
         struct ca_slot_info info;
+        system_ids_t *p_ids;
+        ca_msg_t ca_msg;
         info.num = 0;
 
         /* We don't reset the CAM in that case because it's done by the
@@ -1977,11 +2003,16 @@ void en50221_Reset( void )
         }
 
         /* Allocate a dummy sessions */
-        p_sessions[ 0 ].i_resource_id = RI_CONDITIONAL_ACCESS_SUPPORT;
+        p_sessions[0].i_resource_id = RI_CONDITIONAL_ACCESS_SUPPORT;
+        p_sessions[0].pf_close = ConditionalAccessClose;
+        if ( p_sessions[0].p_sys == NULL )
+            p_sessions[0].p_sys = malloc(sizeof(system_ids_t));
+        memset( p_sessions[0].p_sys, 0, sizeof(system_ids_t) );
+        p_ids = (system_ids_t *)p_sessions[0].p_sys;
+        p_ids->b_high_level = 1;
 
         /* Get application info to find out which cam we are using and make
            sure everything is ready to play */
-        ca_msg_t ca_msg;
         ca_msg.length=3;
         ca_msg.msg[0] = ( AOT_APPLICATION_INFO & 0xFF0000 ) >> 16;
         ca_msg.msg[1] = ( AOT_APPLICATION_INFO & 0x00FF00 ) >> 8;
